package com.supervision.police.dto.caseScore;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.constant.ScoreEnum;
import com.supervision.police.domain.*;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.ai.ollama.OllamaChatClient;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 案件得分详情
 */
@Data
public class CaseScoreDetailBuilder {

    @Schema(description = "案件id")
    private String caseId;

    @Schema(description = "案件名称")
    private String caseName;

    @Schema(description = "当事人列表")
    private List<LawParty> lawPartyList;

    @Schema(description = "指标结果")
    private List<IndexResult> indexResultList;

    /**
     * Ollama 客户端
     */
    private OllamaChatClient chatClient;


    /**
     * 共性指标分数
     * @return
     */
    public Integer getCommonScore() {

        return sumIndexGroupScore("1");
    }

    /**
     * 入罪指标分数
     * @return
     */
    public Integer getCrimeScore() {

        return sumIndexGroupScore("2");
    }

    /**
     * 出罪指标分数
     * @return
     */
    public Integer getCrimeOutScore() {
        return sumIndexGroupScore("3");
    }

    /**
     * 获取行为人
     */
    public LawParty getLawActor() {
        if (CollUtil.isEmpty(this.lawPartyList)){
            return null;
        }
        return this.lawPartyList.stream()
                .filter(lawParty -> StrUtil.equals("1",lawParty.getUserType())).findFirst().orElse(null);
    }

    public void setIndexResultList(List<ModelIndex> indexList, List<ModelIndexResult> indexResultList,
                                   List<ModelAtomicIndex> atomicIndexList, List<ModelAtomicResult> atomicResultList) {

        if (CollUtil.isEmpty(indexList) || CollUtil.isEmpty(indexResultList)){
            this.indexResultList = new ArrayList<>();
            return;
        }

        Map<String, ModelIndexResult> modelIndexResultMap = indexResultList.stream().collect(
                Collectors.toMap(ModelIndexResult::getIndexId, modelIndexResult -> modelIndexResult));

       this.indexResultList = indexList.stream().map(indexInfo ->
               new IndexResult(indexInfo,modelIndexResultMap.get(indexInfo.getId()),atomicIndexList,atomicResultList)
       ).toList();


    }

    /**
     * 设置行为人
     * @param casePersonList
     */
    public void setLawActor(List<CasePerson> casePersonList) {
        if (CollUtil.isEmpty(casePersonList)){
            this.lawPartyList = new ArrayList<>();
            return;
        }
        this.lawPartyList = casePersonList.stream()
                .map(casePerson -> {
                    LawParty lawParty = new LawParty();
                    lawParty.setUserId(casePerson.getId());
                    lawParty.setUserName(casePerson.getName());
                    lawParty.setUserType(casePerson.getRoleCode());
                    return lawParty;
                }).toList();
    }


    /**
     * 构建CaseScoreDetailDTO
     */
    public CaseScoreDetailDTO build(){

        CaseScoreDetailDTO caseScoreDetailDTO = new CaseScoreDetailDTO();
        caseScoreDetailDTO.setCaseName(this.caseName);
        caseScoreDetailDTO.setCommonScore(this.getCommonScore());
        Integer crimeScore = this.getCrimeScore();
        Integer crimeOutScore = this.getCrimeOutScore();
        caseScoreDetailDTO.setCrimeScore(crimeScore < crimeOutScore? crimeOutScore:crimeScore);

        int sumScore = caseScoreDetailDTO.getCommonScore() + caseScoreDetailDTO.getCrimeScore();
        ScoreEnum scoreEnum = ScoreEnum.getScoreEnum(sumScore);
        if (scoreEnum != null){
            caseScoreDetailDTO.setScoreDesc(scoreEnum.getDesc());
            caseScoreDetailDTO.setJudicialCharacterization(scoreEnum.getName());
        }

        caseScoreDetailDTO.setCommonIndexCount(this.countIndexGroup("1",res->true).intValue());
        caseScoreDetailDTO.setCommonIndexHitCount(this.countIndexGroup("1",res->res.getActualScore() > 0).intValue());

        if (crimeScore < crimeOutScore){
            caseScoreDetailDTO.setCrimeIndexCount(this.countIndexGroup("3",res->true).intValue());
            caseScoreDetailDTO.setCrimeIndexHitCount(this.countIndexGroup("3",res->res.getActualScore() > 0).intValue());
        }else {
            caseScoreDetailDTO.setCrimeIndexCount(this.countIndexGroup("2",res->true).intValue());
            caseScoreDetailDTO.setCrimeIndexHitCount(this.countIndexGroup("2",res->res.getActualScore() > 0).intValue());
        }
        return caseScoreDetailDTO;
    }

    private Integer sumIndexGroupScore(String indexGroup) {
        if (CollUtil.isEmpty(this.indexResultList)){
            return 0;
        }

        return this.indexResultList.stream()
                .filter(indexResult -> StrUtil.equals(indexGroup,indexResult.getIndexGroup()))
                .mapToInt(IndexResult::getIndexScore).sum();

    }

    private Long countIndexGroup(String indexGroup, Function<IndexResult, Boolean> function) {
        if (CollUtil.isEmpty(this.indexResultList)){
            return 0L;
        }

        return this.indexResultList.stream()
                .filter(indexResult -> StrUtil.equals(indexGroup,indexResult.getIndexGroup())
                        && function.apply(indexResult)).count();

    }
}
